package dev.fyloz.colorrecipesexplorer.logic.files

import dev.fyloz.colorrecipesexplorer.utils.FilePath
import org.springframework.core.io.Resource
import org.springframework.core.io.ResourceLoader
import org.springframework.stereotype.Service

@Service
class ResourceFileLogic(
    private val resourceLoader: ResourceLoader
) : FileLogic {
    override fun exists(path: String) =
        fullPath(path).resource.exists()

    override fun read(path: String): Resource =
        fullPath(path).resource.also {
            if (!it.exists()) {
                throw FileNotFoundException(path)
            }
        }

    override fun listDirectoryFiles(path: String): Collection<CachedFile> {
        val content = fullPath(path).resource.file.listFiles() ?: return setOf()

        return content
            .filterNotNull()
            .filter { it.isFile }
            .map { it.toFileSystemItem() as CachedFile }
    }

    override fun fullPath(path: String) =
        FilePath("classpath:${path}")

    val FilePath.resource: Resource
        get() = resourceLoader.getResource(this.value)
}
