package dev.fyloz.colorrecipesexplorer.model

import dev.fyloz.colorrecipesexplorer.dtos.MaterialTypeDto
import org.hibernate.annotations.ColumnDefault
import javax.persistence.*

@Entity
@Table(name = "material_type")
data class MaterialType(
    @Id
    @GeneratedValue(strategy = GenerationType.IDENTITY)
    override val id: Long? = null,

    @Column(unique = true)
    val name: String = "",

    @Column(unique = true)
    val prefix: String = "",

    @Column(name = "use_percentages")
    @ColumnDefault("false")
    val usePercentages: Boolean = false,

    @Column(name = "system_type")
    @ColumnDefault("false")
    val systemType: Boolean = false
) : ModelEntity

// ==== DSL ====
fun materialType(
    id: Long? = null,
    name: String = "name",
    prefix: String = "PRE",
    usePercentages: Boolean = false,
    systemType: Boolean = false,
    op: MaterialType.() -> Unit = {}
) = MaterialType(id, name, prefix, usePercentages, systemType).apply(op)

fun materialType(
    materialType: MaterialType,
    newId: Long? = null,
    newName: String? = null,
    newSystemType: Boolean? = null
) = with(materialType) {
    MaterialType(
        newId ?: id,
        newName ?: name,
        prefix,
        usePercentages,
        newSystemType ?: systemType
    )
}

@Deprecated("Temporary DSL for transition")
fun materialType(
    dto: MaterialTypeDto
) = MaterialType(dto.id, dto.name, dto.prefix, dto.usePercentages, dto.systemType)

@Deprecated("Temporary DSL for transition")
fun materialTypeDto(
    entity: MaterialType
) = MaterialTypeDto(entity.id!!, entity.name, entity.prefix, entity.usePercentages, entity.systemType)